@@ -33,18 +33,67 @@ if(!IsAdmin())
 }
 
 //****************************************************************************
-// addLink -- adding a link
-//****************************************************************************
-function addLink($newLink)
+function editLink($lid,$newLink,$newSite)
 {
-    if(!empty($newLink)){
-        global $cfg;
-        addNewLink($newLink);
-        AuditAction($cfg["constants"]["admin"], "New "._LINKS_MENU.": ".$newLink);
+    global $cfg;
+
+    if(!empty($newLink))
+    {
+        if(strpos($newLink, "http://" ) !== 0 && strpos($newLink, "https://" ) !== 0 && strpos($newLink, "ftp://" ) !== 0)
+        {
+            $newLink = "http://".$newLink;
+        }
+        empty($newSite) && $newSite = $newLink;
+
+        $oldLink = getLink($lid);
+        $oldSite = getSite($lid);
+        alterLink($lid, $newLink, $newSite);
+        AuditAction($cfg["constants"]["admin"], "Change Link: ".$oldSite." [".$oldLink."] -> ".$newSite." [".$newLink."]");
     }
     header("location: admin.php?op=editLinks");
 }
 
+//****************************************************************************
+// addLink -- adding a link
+//****************************************************************************
+function addLink($newLink,$newSite)
+{
+    if(!empty($newLink))
+    {
+        if(strpos($newLink, "http://" ) !== 0 && strpos($newLink, "https://" ) !== 0 && strpos($newLink, "ftp://" ) !== 0)
+        {
+            $newLink = "http://".$newLink;
+        }
+        empty($newSite) && $newSite = $newLink;
+        global $cfg;
+        addNewLink($newLink, $newSite);
+        AuditAction($cfg["constants"]["admin"], "New "._LINKS_MENU.": ".$newSite." [".$newLink."]");
+    }
+    header("location: admin.php?op=editLinks");
+}
+
+//****************************************************************************
+// moveLink -- moving a link up or down in the list of links
+//****************************************************************************
+function moveLink($lid, $direction)
+{
+    global $db, $cfg;
+    if  (!isset($lid) && !isset($direction)&& $direction !== "up" && $direction !== "down" )
+    {
+        header("location: admin.php?op=editLinks");
+    }
+    $idx = getLinkSortOrder($lid);
+    $position = array("up"=>-1, "down"=>1);
+    $new_idx = $idx+$position[$direction];
+    $sql = "UPDATE tf_links SET sort_order=".$idx." WHERE sort_order=".$new_idx;
+    $db->Execute($sql);
+    showError($db, $sql);
+    $sql = "UPDATE tf_links SET sort_order=".$new_idx." WHERE lid=".$lid;
+    $db->Execute($sql);
+    showError($db, $sql);
+    header("Location: admin.php?op=editLinks");
+}
+
 //****************************************************************************
 // addRSS -- adding a RSS link
 //****************************************************************************
@@ -124,7 +173,7 @@ function updateUser($user_id, $org_user_id, $pass1, $userType, $hideOffline)
 function deleteLink($lid)
 {
     global $cfg;
-    AuditAction($cfg["constants"]["admin"], _DELETE." Link: ".getLink($lid));
+    AuditAction($cfg["constants"]["admin"], _DELETE." Link: ".getSite($lid)." [".getLink($lid)."]");
     deleteOldLink($lid);
     header("location: admin.php?op=editLinks");
 }
@@ -322,7 +371,7 @@ function displayActivity($min=0, $user="", $srchFile="", $srchAction="")
             $user_icon = "images/user.gif";
         }
 
-        $ip_info = $ip_resolved."<br>".$user_agent;
+        $ip_info = htmlentities($ip_resolved, ENT_QUOTES)."<br>".htmlentities($user_agent, ENT_QUOTES);
 
         $output .= "<tr>";
         if (IsUser($user_id))
@@ -333,11 +382,11 @@ function displayActivity($min=0, $user="", $srchFile="", $srchAction="")
         {
             $output .= "<td><img src=\"".$user_icon."\" width=17 height=14 title=\"n/a\" border=0 align=\"bottom\">".$user_id."&nbsp;&nbsp;</td>";
         }
-        $output .= "<td><div class=\"tiny\">".$action."</div></td>";
+        $output .= "<td><div class=\"tiny\">".htmlentities($action, ENT_QUOTES)."</div></td>";
         $output .= "<td><div align=center><div class=\"tiny\" align=\"left\">";
-        $output .= $file;
+        $output .= htmlentities($file, ENT_QUOTES);
         $output .= "</div></td>";
-        $output .= "<td><div class=\"tiny\" align=\"left\"><a href=\"javascript:void(0)\" onclick=\"return overlib('".$ip_info."<br>', STICKY, CSSCLASS);\" onmouseover=\"return overlib('".$ip_info."<br>', CSSCLASS);\" onmouseout=\"return nd();\"><img src=\"images/properties.png\" width=\"18\" height=\"13\" border=\"0\"><font class=tiny>".$ip."</font></a></div></td>";
+        $output .= "<td><div class=\"tiny\" align=\"left\"><a href=\"javascript:void(0)\" onclick=\"return overlib('".$ip_info."<br>', STICKY, CSSCLASS);\" onmouseover=\"return overlib('".$ip_info."<br>', CSSCLASS);\" onmouseout=\"return nd();\"><img src=\"images/properties.png\" width=\"18\" height=\"13\" border=\"0\"><font class=tiny>".htmlentities($ip, ENT_QUOTES)."</font></a></div></td>";
         $output .= "<td><div class=\"tiny\" align=\"center\">".date(_DATETIMEFORMAT, $time)."</div></td>";
         $output .= "</tr>";
 
@@ -386,16 +435,19 @@ function displayActivity($min=0, $user="", $srchFile="", $srchAction="")
         <option value="">-- <?php echo _ALL ?> --</option>
 <?php
         $selected = "";
-        foreach ($cfg["constants"] as $action)
+        if(is_array($cfg["constants"] ))
         {
-            $selected = "";
-            if($action != $cfg["constants"]["hit"])
+            foreach ($cfg["constants"] as $action)
             {
-                if($srchAction == $action)
+                $selected = "";
+                if($action != $cfg["constants"]["hit"])
                 {
-                    $selected = "selected";
+                    if($srchAction == $action)
+                    {
+                        $selected = "selected";
+                    }
+                    echo "<option value=\"".htmlentities($action, ENT_QUOTES)."\" ".$selected.">".htmlentities($action, ENT_QUOTES)."</option>";
                 }
-                echo "<option value=\"".$action."\" ".$selected.">".$action."</option>";
             }
         }
 ?>
@@ -413,7 +465,7 @@ function displayActivity($min=0, $user="", $srchFile="", $srchAction="")
             {
                 $selected = "selected";
             }
-            echo "<option value=\"".$users[$inx]."\" ".$selected.">".$users[$inx]."</option>";
+            echo "<option value=\"".htmlentities($users[$inx], ENT_QUOTES)."\" ".$selected.">".htmlentities($users[$inx], ENT_QUOTES)."</option>";
         }
 ?>
         </select>
@@ -935,27 +987,70 @@ function editLinks()
     displayMenu();
     echo "<div align=\"center\">";
     echo "<table border=1 bordercolor=\"".$cfg["table_admin_border"]."\" cellpadding=\"2\" cellspacing=\"0\" bgcolor=\"".$cfg["table_data_bg"]."\">";
-    echo "<tr><td bgcolor=\"".$cfg["table_header_bg"]."\" background=\"themes/".$cfg["theme"]."/images/bar.gif\">";
+    echo "<tr><td colspan=\"2\" bgcolor=\"".$cfg["table_header_bg"]."\" background=\"themes/".$cfg["theme"]."/images/bar.gif\">";
     echo "<img src=\"images/properties.png\" width=18 height=13 border=0>&nbsp;&nbsp;<font class=\"title\">"._ADMINEDITLINKS."</font>";
-    echo "</td></tr><tr><td align=\"center\">";
+    echo "</td></tr><tr><td colspan=2 align=\"center\">";
 ?>
     <form action="admin.php?op=addLink" method="post">
     <?php echo _FULLURLLINK ?>:
-    <input type="Text" size="50" maxlength="255" name="newLink">
+    <input type="Text" size="30" maxlength="255" name="newLink">
+    Site Name:
+    <input type="Text" size="30" maxlength="255" name="newSite">
     <input type="Submit" value="<?php echo _UPDATE ?>"><br>
     </form>
 <?php
     echo "</td></tr>";
     $arLinks = GetLinks();
-    $arLid = Array_Keys($arLinks);
-    $inx = 0;
-    foreach($arLinks as $link)
-    {
-        $lid = $arLid[$inx++];
-        echo "<tr><td><a href=\"admin.php?op=deleteLink&lid=".$lid."\"><img src=\"images/delete_on.gif\" width=16 height=16 border=0 title=\""._DELETE." ".$lid."\" align=\"absmiddle\"></a>&nbsp;";
-        echo "<a href=\"".$link."\" target=\"_blank\">".$link."</a></td></tr>\n";
-    }
 
+    if (is_array($arLinks))
+    {
+        $arLid = Array_Keys($arLinks);
+        $inx = 0;
+        $link_count = count($arLinks);
+
+        foreach($arLinks as $link)
+        {
+            $lid = $arLid[$inx++];
+            $ed = getRequestVar("edit");
+            if (!empty($ed) && $ed == $link['lid'])
+            {
+                echo "<tr><td colspan=\"2\">";
+    ?>
+      <form action="admin.php?op=editLink" method="post">
+      <?php echo _FULLURLLINK ?>:
+      <input type="Text" size="30" maxlength="255" name="editLink" value="<?php echo $link['url'] ?>">
+      Site Name:
+      <input type="Text" size="30" maxlength="255" name="editSite" value="<?php echo $link['sitename'] ?>">
+      <input type="hidden" name="lid" value="<?php echo $lid ?>">
+      <input type="Submit" value="<?php echo _UPDATE ?>"><br>
+      </form>
+    <?php
+            }
+            else
+            {
+                echo "<tr><td>";
+                echo "<a href=\"admin.php?op=deleteLink&lid=".$lid."\"><img src=\"images/delete_on.gif\" width=16 height=16 border=0 title=\""._DELETE." ".$lid."\" align=\"absmiddle\"></a>&nbsp;";
+                echo "<a href=\"admin.php?op=editLinks&edit=".$lid."\"><img src=\"images/properties.png\" width=18 height=13 border=0 title=\""._EDIT." ".$lid."\" align=\"absmiddle\"></a>&nbsp;";
+                echo "<a href=\"".$link['url']."\" target=\"_blank\">".$link['sitename']."</a></td>\n";
+                echo "<td align=center width='36'>";
+
+                if ($inx > 1 ){
+                    // Only put an 'up' arrow if this isn't the first entry:
+                    echo "<a href='admin.php?op=moveLink&amp;direction=up&amp;lid=".$lid."'>";
+                    echo "<img src='images/uparrow.png' width='16' height='16' ";
+                    echo "border='0' title='Move link up' align='absmiddle' alt='Up'></a>";
+                }
+
+                if ($inx != count($arLinks)) {
+                    // Only put a 'down' arrow if this isn't the last item:
+                    echo "<a href='admin.php?op=moveLink&amp;direction=down&amp;lid=".$lid."'>";
+                    echo "<img src='images/downarrow.png' width='16' height='16' ";
+                    echo "border='0' title='Move link down' align='absmiddle' alt='Down'></a>";
+                }
+                echo "</td></tr>";
+            }
+        }
+    }
     echo "</table></div><br><br><br>";
 
     DisplayFoot();
@@ -989,13 +1084,15 @@ function editRSS()
     $arLinks = GetRSSLinks();
     $arRid = Array_Keys($arLinks);
     $inx = 0;
-    foreach($arLinks as $link)
+    if(is_array($arLinks))
     {
-        $rid = $arRid[$inx++];
-        echo "<tr><td><a href=\"admin.php?op=deleteRSS&rid=".$rid."\"><img src=\"images/delete_on.gif\" width=16 height=16 border=0 title=\""._DELETE." ".$rid."\" align=\"absmiddle\"></a>&nbsp;";
-        echo "<a href=\"".$link."\" target=\"_blank\">".$link."</a></td></tr>\n";
+        foreach($arLinks as $link)
+        {
+            $rid = $arRid[$inx++];
+            echo "<tr><td><a href=\"admin.php?op=deleteRSS&rid=".$rid."\"><img src=\"images/delete_on.gif\" width=16 height=16 border=0 title=\""._DELETE." ".$rid."\" align=\"absmiddle\"></a>&nbsp;";
+            echo "<a href=\"".$link."\" target=\"_blank\">".$link."</a></td></tr>\n";
+        }
     }
-
     echo "</table></div><br><br><br>";
 
     DisplayFoot();
@@ -1453,6 +1550,31 @@ function configSettings()
                 <input name="page_refresh" type="Text" maxlength="3" value="<?php    echo($cfg["page_refresh"]); ?>" size="3">
             </td>
         </tr>
+<?php
+    if (!defined("IMG_JPG")) define("IMG_JPG", 2);
+    // Check gd is loaded AND that jpeg image type is supported:
+    if (extension_loaded('gd') && (imagetypes() & IMG_JPG))
+    {
+?>
+        <tr>
+            <td align="left" width="350" valign="top"><strong>Enable Security Code Login</strong><br>
+            Requires users to enter a security code from a generated graphic to login (if enabled automated logins will NOT work):
+            </td>
+            <td valign="top">
+                <select name="security_code">
+                        <option value="1">true</option>
+                        <option value="0" <?php
+                            if (!$cfg["security_code"])
+                            {
+                                echo "selected";
+                            }
+                        ?>>false</option>
+                </select>
+            </td>
+        </tr>
+<?php
+    }
+?>
         <tr>
             <td align="left" width="350" valign="top"><strong>Default Theme</strong><br>
             Select the default theme that users will have (including login screen):
@@ -1558,21 +1680,22 @@ function configSettings()
 function updateConfigSettings()
 {
     global $cfg;
-    
+
     $tmpPath = getRequestVar("path");
-    if (!empty($tmpPath) && substr( $_POST["path"], -1 )  != "/")
+    
+    if (!empty($tmpPath) && substr( $tmpPath, -1 )  != "/")
     {
         // path requires a / on the end
         $_POST["path"] = $_POST["path"] . "/";
     }
     
-    if ($_POST["AllowQueing"] != $cfg["AllowQueing"] ||
-        $_POST["maxServerThreads"] != $cfg["maxServerThreads"] ||
-        $_POST["maxUserThreads"] != $cfg["maxUserThreads"] ||
-        $_POST["sleepInterval"] != $cfg["sleepInterval"] ||
-        $_POST["debugTorrents"] != $cfg["debugTorrents"] ||
-        $_POST["tfQManager"] != $cfg["tfQManager"] ||
-        $_POST["btphpbin"] != $cfg["btphpbin"]
+    if ((array_key_exists("AllowQueing",$_POST) && $_POST["AllowQueing"] != $cfg["AllowQueing"]) ||
+        (array_key_exists("maxServerThreads",$_POST) && $_POST["maxServerThreads"] != $cfg["maxServerThreads"]) ||
+        (array_key_exists("maxUserThreads",$_POST) && $_POST["maxUserThreads"] != $cfg["maxUserThreads"]) ||
+        (array_key_exists("sleepInterval",$_POST) && $_POST["sleepInterval"] != $cfg["sleepInterval"]) ||
+        (array_key_exists("debugTorrents",$_POST) && $_POST["debugTorrents"] != $cfg["debugTorrents"]) ||
+        (array_key_exists("tfQManager",$_POST) && $_POST["tfQManager"] != $cfg["tfQManager"]) ||
+        (array_key_exists("btphpbin",$_POST) && $_POST["btphpbin"] != $cfg["btphpbin"])
         )
     {
         // kill QManager process;
@@ -1587,11 +1710,11 @@ function updateConfigSettings()
             AuditAction($cfg["constants"]["admin"], " Updating TorrentFlux Settings");
 
         // if enabling Start QManager
-        if($_POST["AllowQueing"])
+        if($cfg["AllowQueing"])
         {
             sleep(2);
-            startQManager($_POST["maxServerThreads"], $_POST["maxUserThreads"], $_POST["sleepInterval"]);
-        sleep(1);
+            startQManager($cfg["maxServerThreads"], $cfg["maxUserThreads"], $cfg["sleepInterval"]);
+            sleep(1);
         }
     }
     else
@@ -1726,16 +1849,16 @@ function queueSettings()
                         <option value="0" <?php
             if (array_key_exists("debugTorrents",$cfg))
             {
-				if (!$cfg["debugTorrents"])
-				{
-				    echo "selected";
-				}
-			}
-			else
-			{
-				insertSetting("debugTorrents",false);
-				echo "selected";
-			}
+                if (!$cfg["debugTorrents"])
+                {
+                    echo "selected";
+                }
+            }
+            else
+            {
+                insertSetting("debugTorrents",false);
+                echo "selected";
+            }
                         ?>>false</option>
                     </select>
                 </td>
@@ -1872,12 +1995,14 @@ function queueSettings()
 
         // get running torrents and List them out.
         $runningTorrents = getRunningTorrents();
-        foreach ($runningTorrents as $key => $value)
+        if(is_array($runningTorrents))
         {
-            $rt = new RunningTorrent($value);
-            $output .= $rt->BuildAdminOutput();
+            foreach ($runningTorrents as $key => $value)
+            {
+                $rt = new RunningTorrent($value);
+                $output .= $rt->BuildAdminOutput();
+            }
         }
-
         if( strlen($output) == 0 )
         {
             $output = "<tr><td colspan=3><div class=\"tiny\" align=center>No Running Torrents</div></td></tr>";
@@ -1994,7 +2119,7 @@ function searchSettings()
             <td valign="top">
 <?php
                 echo "<select multiple name=\"".$sEngine->catFilterName."[]\" size=\"8\" STYLE=\"width: 125px\">";
-                echo "<option value=\"\">[NO FILTER]</option>";
+                echo "<option value=\"-1\">[NO FILTER]</option>";
                 foreach ($sEngine->getMainCategories(false) as $mainId => $mainName)
                 {
                     echo "<option value=\"".$mainId."\" ";
@@ -2090,13 +2215,27 @@ switch ($op)
         deleteRSS($rid);
     break;
 
+    case "editLink":
+        $lid = getRequestVar('lid');
+        $editLink = getRequestVar('editLink');
+        $editSite = getRequestVar('editSite');
+        editLink($lid, $editLink, $editSite);
+    break;
+
     case "editLinks":
         editLinks();
     break;
 
     case "addLink":
         $newLink = getRequestVar('newLink');
-        addLink($newLink);
+        $newSite = getRequestVar('newSite');
+        addLink($newLink,$newSite);
+    break;
+
+    case "moveLink":
+        $lid = getRequestVar('lid');
+        $direction = getRequestVar('direction');
+        moveLink($lid, $direction);
     break;
 
     case "deleteLink":
@@ -2164,4 +2303,4 @@ switch ($op)
 //****************************************************************************
 //****************************************************************************
 
-?>
\ No newline at end of file
+?>
