@@ -116,7 +116,7 @@ if ($down != "" && $cfg["enable_file_download"])
             header("Content-length: " . file_size($path) . "\n");
 
             // write the session to close so you can continue to browse on the site.
-            session_write_close("TorrentFlux");
+            session_write_close();
 
             //$fp = fopen($path, "r");
             $fp = popen("cat \"$path\"", "r");
@@ -195,7 +195,7 @@ if ($tar != "" && $cfg["enable_file_download"])
             header('Content-Disposition: attachment; filename="'.$sendname.'.'.$cfg["package_type"].'"');
 
             // write the session to close so you can continue to browse on the site.
-            session_write_close("TorrentFlux");
+            session_write_close();
 
             // Make it a bit easier for tar/zip.
             chdir(dirname($tar));
@@ -246,6 +246,19 @@ function ConfirmDelete(file)
 {
     return confirm("<?php echo _ABOUTTODELETE ?>: " + file)
 }
+
+function checkCheck(thisIn)
+{
+    var form = thisIn.form, i = 0;
+    for(i=0; i < form.length; i++)
+    {
+        if(form[i].type == 'checkbox' && form[i].name != 'checkall')
+        {
+            form[i].checked = thisIn.checked;
+        }
+    }
+}
+
 </script>
 
 <?php
@@ -254,7 +267,15 @@ displayDriveSpaceBar(getDriveSpace($cfg["path"]));
 echo "<br>";
 
 if(!isset($dir)) $dir = "";
-ListDirectory($cfg["path"].$dir);
+
+if (!file_exists($cfg["path"].$dir))
+{
+    echo "<strong>".htmlentities($dir)."</strong> could not be found or is not valid.";
+}
+else
+{
+    ListDirectory($cfg["path"].$dir);
+}
 
 DisplayFoot();
 
@@ -285,10 +306,18 @@ function ListDirectory($dirName)
             $parentURL="dir.php?dir=" . urlencode($matches[1]);
         }
 
+        echo "<table cellpadding=2 width=740>";
+        echo '<form action="multi.php" method="post" name="multidir">';
+        echo '<input type="hidden" name="action" value="fileDelete" />';
+        echo '<tr><td colspan="2">';
         echo "<a href=\"" . $parentURL . "\"><img src=\"images/up_dir.gif\" width=16 height=16 title=\""._BACKTOPARRENT."\" border=0>["._BACKTOPARRENT."]</a>";
+        echo '</td>';
+        echo '<td align="right">Multi-Delete-&gt;</td>';
+        echo '<td align="right"><a href="javascript:document.multidir.submit()" onclick="return ConfirmDelete(\'Multiple Files\')">';
+        echo '<img src="images/delete_on.gif" title="Delete Multiple Files" border="0" height="16" width="16"></a>';
+        echo '<input type="checkbox" onclick="checkCheck(this);" /></td></tr>';
     }
 
-    echo "<table cellpadding=2 width=740>";
     $handle = opendir($dirName);
     while($entry = readdir($handle))
     {
@@ -306,17 +335,17 @@ function ListDirectory($dirName)
                 echo "<td>&nbsp;</td>";
                 echo "<td>&nbsp;</td>";
                 echo "<td align=\"right\">";
-                
+
                 if ($cfg["enable_maketorrent"])
                 {
                     echo "<a href=\"JavaScript:MakeTorrent('maketorrent.php?path=".urlencode($dir.$entry)."')\"><img src=\"images/make.gif\" width=16 height=16 title=\"Make Torrent\" border=0></a>";
                 }
-                
+
                 if ($cfg["enable_file_download"])
                 {
                     echo "<a href=\"dir.php?tar=".urlencode($dir.$entry)."\"><img src=\"images/tar_down.gif\" width=16 height=16 title=\"Download as ".$cfg["package_type"]."\" border=0></a>";
                 }
-                
+
                 // The following lines of code were suggested by Jody Steele jmlsteele@stfu.ca
                 // this is so only the owner of the file(s) or admin can delete
                 // only give admins and users who "own" this directory
@@ -324,6 +353,7 @@ function ListDirectory($dirName)
                 if(IsAdmin($cfg["user"]) || preg_match("/^" . $cfg["user"] . "/",$dir))
                 {
                     echo "<a href=\"dir.php?del=".urlencode($dir.$entry)."\" onclick=\"return ConfirmDelete('".addslashes($entry)."')\"><img src=\"images/delete_on.gif\" width=16 height=16 title=\""._DELETE."\" border=0></a>";
+                    echo "<input type=\"checkbox\" name=\"file[]\" value=\"".urlencode($dir.$entry)."\">";
                 }
                 else
                 {
@@ -417,7 +447,7 @@ function ListDirectory($dirName)
                 {
                     echo "<a href=\"JavaScript:MakeTorrent('maketorrent.php?path=".urlencode($dir.$entry)."')\"><img src=\"images/make.gif\" width=16 height=16 title=\"Make Torrent\" border=0></a>";
                 }
-                
+
                 if ($cfg["enable_file_download"])
                 {
                     // Show the download button
@@ -433,6 +463,7 @@ function ListDirectory($dirName)
                 if(IsAdmin($cfg["user"]) || preg_match("/^" . $cfg["user"] . "/",$dir))
                 {
                     echo "<a href=\"dir.php?del=".urlencode($dir.$entry)."\" onclick=\"return ConfirmDelete('".addslashes($entry)."')\"><img src=\"images/delete_on.gif\" width=16 height=16 title=\""._DELETE."\" border=0></a>";
+                    echo "<input type=\"checkbox\" name=\"file[]\" value=\"".urlencode($dir.$entry)."\">";
                 }
                 else
                 {
@@ -498,4 +529,4 @@ function getExtension($fileName)
     return strtolower($ext);
 }
 
-?>
\ No newline at end of file
+?>
