@@ -23,7 +23,8 @@
 */
 
 // Start Session and grab user
-session_start("TorrentFlux");
+session_name("TorrentFlux");
+session_start();
 
 if(isset($_SESSION['user']))
 {
@@ -61,6 +62,19 @@ checkTorrentPath();
 // START FUNCTIONS HERE
 //**********************************************************************************
 
+//*********************************************************
+function getLinkSortOrder($lid)
+{
+    global $db;
+
+    // Get Current sort order index of link with this link id:
+    $sql="SELECT sort_order FROM tf_links WHERE lid=$lid";
+    $rtnValue=$db->GetOne($sql);
+    showError($db,$sql);
+
+    return $rtnValue;
+}
+
 //*********************************************************
 // avddelete()
 function avddelete($file)
@@ -352,7 +366,7 @@ function resetOwner($file)
             $rtnValue = GetSuperAdmin();
         }
 
-        $host_resolved = gethostbyaddr($cfg['ip']);
+        $host_resolved = $cfg['ip'];
         $create_time = time();
 
         $rec = array(
@@ -389,6 +403,19 @@ function getCookie($cid)
     return $rtnValue[0];
 }
 
+//*********************************************************
+function getAllCookies($uid)
+{
+    global $cfg, $db;
+
+    $rtnValue = "";
+
+    $sql = "SELECT c.cid, c.host, c.data FROM tf_cookies AS c, tf_users AS u WHERE u.uid=c.uid AND u.user_id='" . $uid . "' order by host";
+    $rtnValue = $db->GetAll($sql);
+
+    return $rtnValue;
+}
+
 // ***************************************************************************
 // Delete Cookie Host Information
 function deleteCookieInfo($cid)
@@ -407,7 +434,7 @@ function addCookieInfo( $newCookie )
     // Get uid of user
     $sql = "SELECT uid FROM tf_users WHERE user_id = '" . $cfg["user"] . "'";
     $uid = $db->GetOne( $sql );
-    $sql = "INSERT INTO tf_cookies ( cid, uid, host, data ) VALUES ( '', '" . $uid . "', '" . $newCookie["host"] . "', '" . $newCookie["data"] . "' )";
+    $sql = "INSERT INTO tf_cookies ( uid, host, data ) VALUES ( " . $uid . ", " . $db->qstr($newCookie["host"]) . ", " . $db->qstr($newCookie["data"]) . " )";
     $db->Execute( $sql );
     showError( $db, $sql );
 }
@@ -417,11 +444,24 @@ function addCookieInfo( $newCookie )
 function modCookieInfo($cid, $newCookie)
 {
     global $db;
-    $sql = "UPDATE tf_cookies SET host='" . $newCookie["host"] . "', data='" . $newCookie["data"] . "' WHERE cid='" . $cid . "'";
+    $sql = "UPDATE tf_cookies SET host='" . $newCookie["host"] . "', data='" . $newCookie["data"] . "' WHERE cid=" . $cid;
     $db->Execute($sql);
     showError($db,$sql);
 }
 
+//*********************************************************
+function getSite($lid)
+{
+    global $cfg, $db;
+
+    $rtnValue = "";
+
+    $sql = "SELECT sitename FROM tf_links WHERE lid=".$lid;
+    $rtnValue = $db->GetOne($sql);
+
+    return $rtnValue;
+}
+
 //*********************************************************
 function getLink($lid)
 {
@@ -566,10 +606,14 @@ function GetMessage($mid)
 {
     global $cfg, $db;
 
-    $sql = "select from_user, message, ip, time, isnew, force_read from tf_messages where mid=".$mid." and to_user=".$db->qstr($cfg['user']);
+    $rtnValue = array();
 
-    $rtnValue = $db->GetRow($sql);
-    showError($db,$sql);
+    if (is_numeric($mid))
+    {
+        $sql = "select from_user, message, ip, time, isnew, force_read from tf_messages where mid=".$mid." and to_user=".$db->qstr($cfg['user']);
+        $rtnValue = $db->GetRow($sql);
+        showError($db,$sql);
+    }
 
     return $rtnValue;
 }
@@ -647,7 +691,28 @@ function DeleteMessage($mid)
 function deleteOldLink($lid)
 {
     global $db;
-    $sql = "delete from tf_links where lid=".$lid;
+    // Get Current sort order index of link with this link id:
+    $idx = getLinkSortOrder($lid);
+
+    // Fetch all link ids and their sort orders where the sort order is greater
+    // than the one we're removing - we need to shuffle each sort order down
+    // one:
+    $sql = "SELECT sort_order, lid FROM tf_links ";
+    $sql .= "WHERE sort_order > ".$idx." ORDER BY sort_order ASC";
+    $result = $db->Execute($sql);
+    showError($db,$sql);
+    $arLinks = $result->GetAssoc();
+
+    // Decrement the sort order of each link:
+    foreach($arLinks as $sid => $this_lid)
+    {
+        $sql="UPDATE tf_links SET sort_order=sort_order-1 WHERE lid=".$this_lid;
+        $db->Execute($sql);
+        showError($db,$sql);
+    }
+
+    // Finally delete the link:
+    $sql = "DELETE FROM tf_links WHERE lid=".$lid;
     $result = $db->Execute($sql);
     showError($db,$sql);
 }
@@ -774,13 +839,47 @@ function MarkMessageRead($mid)
     showError($db,$sql);
 }
 
+//**************************************************************************
+// alterLink()
+// This function updates the database and alters the selected links values
+function alterLink($lid,$newLink,$newSite)
+{
+    global $cfg, $db;
+
+    $sql = "UPDATE tf_links SET url='".$newLink."',`sitename`='".$newSite."' WHERE `lid`=".$lid;
+    $db->Execute($sql);
+    showError($db,$sql);
+}
 
 // ***************************************************************************
 // addNewLink - Add New Link
-function addNewLink($newLink)
+function addNewLink($newLink,$newSite)
 {
     global $db;
-    $rec = array('url'=>$newLink);
+    // Link sort order index:
+    $idx = -1;
+
+    // Get current highest link index:
+    $sql = "SELECT sort_order FROM tf_links ORDER BY sort_order DESC";
+    $result = $db->SelectLimit($sql, 1);
+    showError($db, $sql);
+
+    if($result->fields === false)
+    {
+        // No links currently in db:
+        $idx = 0;
+    }
+    else
+    {
+        $idx = $result->fields["sort_order"]+1;
+    }
+
+    $rec = array
+    (
+        'url'=>$newLink,
+        'sitename'=>$newSite,
+        'sort_order'=>$idx
+    );
     $sTable = 'tf_links';
     $sql = $db->GetInsertSql($sTable, $rec);
     $db->Execute($sql);
@@ -871,7 +970,7 @@ function GetLinks()
 
     $link_array = array();
 
-    $link_array = $db->GetAssoc("SELECT lid, url FROM tf_links ORDER BY lid");
+    $link_array = $db->GetAssoc("SELECT lid, url, sitename, sort_order FROM tf_links ORDER BY sort_order");
     return $link_array;
 }
 
@@ -1039,7 +1138,7 @@ function DisplayHead($subTopic, $showButtons=true, $refresh="", $percentdone="")
         <link rel="icon" href="images/favicon.ico" type="image/x-icon" />
         <link rel="shortcut icon" href="images/favicon.ico" type="image/x-icon" />
         <LINK REL="StyleSheet" HREF="themes/<?php echo $cfg["theme"] ?>/style.css" TYPE="text/css">
-        <META HTTP-EQUIV="Pragma" CONTENT="no-cache; charset=<?php echo _CHARSET ?>">
+        <META HTTP-EQUIV="Pragma" CONTENT="no-cache" charset="<?php echo _CHARSET ?>">
 
     <?php
     if ($refresh != "")
@@ -1085,6 +1184,7 @@ function DisplayFoot($showReturn=true)
     if ($showReturn)
     {
         echo "[<a href=\"index.php\">"._RETURNTOTORRENTS."</a>]";
+        echo "</form>";
     }
 ?>
     </div>
@@ -1193,7 +1293,7 @@ function DisplayMessageList()
     echo '<select name="to_user">';
         for($inx = 0; $inx < sizeof($users); $inx++)
         {
-        echo '<option>'.$users[$inx].'</option>';
+        echo '<option>'.htmlentities($users[$inx], ENT_QUOTES).'</option>';
         }
     echo '</select>';
     echo '<input type="Submit" value="' . _COMPOSE .'">';
@@ -1430,6 +1530,8 @@ function FetchTorrent($url)
     ini_set("allow_url_fopen", "1");
     ini_set("user_agent", $_SERVER["HTTP_USER_AGENT"]);
 
+    $rtnValue = "";
+
     $domain  = parse_url( $url );
 
     if( strtolower( substr( $domain["path"], -8 ) ) != ".torrent" )
@@ -1500,13 +1602,14 @@ function FetchTorrent($url)
             $html = FetchHTML( $url, $referer );
 
             // Get the name of the torrent, and make it the filename
-            if( preg_match( "/name([0-9][^:]):(.[^:]+)/i", $html, $html_preg_match ) )
+            if( preg_match( "/name([0-9]+):[^:]+/i", $html, $html_preg_match ) )
             {
                 $filelength = $html_preg_match[1];
-                $filename = $html_preg_match[2];
-                $cfg["save_torrent_name"] = substr( $filename, 0, $filelength ) . ".torrent";
+                $filename = $html_preg_match[0];
+                $cfg["save_torrent_name"] = substr( $filename, 5+strlen($filelength), $filelength ) . ".torrent";
             }
 
+
             // Make sure we have a torrent file
             if( strpos( $html, "d8:" ) === false )
             {
@@ -1556,11 +1659,11 @@ function FetchTorrent($url)
             $html = FetchHTML( $url, $referer );
 
             // Get the name of the torrent, and make it the filename
-            if( preg_match( "/name([0-9][^:]):(.[^:]+)/i", $html, $html_preg_match ) )
+            if( preg_match( "/name([0-9]+):[^:]+/i", $html, $html_preg_match ) )
             {
                 $filelength = $html_preg_match[1];
-                $filename = $html_preg_match[2];
-                $cfg["save_torrent_name"] = substr( $filename, 0, $filelength ) . ".torrent";
+                $filename = $html_preg_match[0];
+                $cfg["save_torrent_name"] = substr( $filename, 5+strlen($filelength), $filelength ) . ".torrent";
             }
 
             if( !empty( $html ) )
@@ -1591,12 +1694,13 @@ function FetchTorrent($url)
     }
     else
     {
+    	$html = substr($html, strpos($html, "d8:"));
         // Get the name of the torrent, and make it the filename
-        if( preg_match( "/name([0-9][^:]):(.[^:]+)/i", $html, $html_preg_match ) )
+        if( preg_match( "/name([0-9]+):[^:]+/i", $html, $html_preg_match ) )
         {
             $filelength = $html_preg_match[1];
-            $filename = $html_preg_match[2];
-            $cfg["save_torrent_name"] = substr( $filename, 0, $filelength ) . ".torrent";
+            $filename = $html_preg_match[0];
+            $cfg["save_torrent_name"] = substr( $filename, 5+strlen($filelength), $filelength ) . ".torrent";
         }
     }
 
@@ -1649,12 +1753,12 @@ function FetchHTML( $url, $referer = "" )
         {
             // Write the outgoing header packet
             // Using required cookie information
-            $packet  = "GET " . $url . "\r\n";
+            $packet  = "GET " . $url . " HTTP/1.0\r\n";
             $packet .= ( !empty( $referer ) ) ? "Referer: " . $referer . "\r\n" : "";
             $packet .= "Accept: */*\r\n";
             $packet .= "Accept-Language: en-us\r\n";
             $packet .= "User-Agent: ".$_SERVER["HTTP_USER_AGENT"]."\r\n";
-            $packet .= "Host: " . $_SERVER["SERVER_NAME"] . "\r\n";
+            $packet .= "Host: " . $domain["host"] . "\r\n";
             $packet .= "Connection: Close\r\n";
             $packet .= "Cookie: " . $cookie . "\r\n\r\n";
 
@@ -1687,7 +1791,9 @@ function FetchHTML( $url, $referer = "" )
     }
 
     // If the HTML is still empty, then try CURL
-    if (($rtnValue == "" && function_exists("curl_init")) || (strpos($rtnValue, "HTTP/1.1 302") > 0 && function_exists("curl_init")))
+    if (($rtnValue == "" && function_exists("curl_init")) ||
+        (strpos($rtnValue, "HTTP/1.0 302") > 0 && function_exists("curl_init")) ||
+        (strpos($rtnValue, "HTTP/1.1 302") > 0 && function_exists("curl_init")))
     {
         // Give CURL a Try
         $ch = curl_init();
@@ -2104,7 +2210,7 @@ function startQManager($maxServerThreads=5,$maxUserThreads=2,$sleepInterval=10)
         }
 
         $btphp = "'" . $cmd1. "; HOME=".$cfg["path"]."; export HOME; nohup " . $pyCmd . " " .$cfg["btphpbin"] . " '";
-        $command = $pyCmd . " " . $cfg["tfQManager"] . " ".$cfg["torrent_file_path"]."queue/ ".$maxServerThreads." ".$maxUserThreads." ".$sleepInterval." ".$btphp." > /dev/null &";
+        $command = $pyCmd . " " . $cfg["tfQManager"] . " ".$cfg["torrent_file_path"]."queue/ ".escapeshellarg($maxServerThreads)." ".escapeshellarg($maxUserThreads)." ".escapeshellarg($sleepInterval)." ".$btphp." > /dev/null &";
         //$command = $pyCmd . " " . $cfg["tfQManager"] . " ".$cfg["torrent_file_path"]."queue/ ".$maxServerThreads." ".$maxUserThreads." ".$sleepInterval." ".$btphp." > /dev/null2>&1 & &";
 
         $result = exec($command);
@@ -2127,7 +2233,7 @@ function stopQManager()
     if($QmgrPID != "")
     {
         AuditAction($cfg["constants"]["QManager"], "Stopping PID:" . $QmgrPID);
-        $result = exec("kill ".$QmgrPID);
+        $result = exec("kill ".escapeshellarg($QmgrPID));
         unlink($cfg["torrent_file_path"] . "queue/tfQManager.pid");
     }
 }
@@ -2140,11 +2246,41 @@ function file_size($file)
     $size = @filesize($file);
     if ( $size == 0)
     {
-        $size = exec("ls -l \"".$file."\" | awk '{print $5}'");
+        $size = exec("ls -l \"".escapeshellarg($file)."\" | awk '{print $5}'");
     }
     return $size;
 }
 
+//**************************************************************************
+// SecurityClean()
+// Cleans the file name for delete and alias file creation
+function SecurityClean($string)
+{
+    global $cfg;
+    
+    if (empty($string))
+    {
+        return $string;
+    }
+    
+    $array = array("<", ">", "\\", "//", "..", "'", "/");
+    foreach ($array as $char)
+    {
+        $string = str_replace($char, NULL, $string);
+    }
+        
+    if( (strtolower( substr( $string, -8 ) ) == ".torrent") || (strtolower( substr( $string, -5 ) ) == ".stat") )
+    {
+        // we are good
+    }
+    else
+    {
+        AuditAction($cfg["constants"]["error"], "Not a stat or torrent: " . $string);
+        die("Invalid file specified.  Action has been logged.");
+    }
+    return $string;
+}
+
 //**************************************************************************
 // getDirList()
 // This method Builds and displays the Torrent Section of the Index Page
