@@ -31,14 +31,14 @@ $messages = "";
 // set refresh option into the session cookie
 if(array_key_exists("pagerefresh", $_GET))
 {
-    if($_GET["pagerefresh"] == "false")
+    if(getRequestVar("pagerefresh") == "false")
     {
         $_SESSION['prefresh'] = false;
         header("location: index.php");
         exit();
     }
 
-    if($_GET["pagerefresh"] == "true")
+    if(getRequestVar("pagerefresh") == "true")
     {
         $_SESSION["prefresh"] = true;
         header("location: index.php");
@@ -114,7 +114,7 @@ if(!empty($torrent))
         $rate = getRequestVar('rate');
         if (empty($rate))
         {
-            if ($rate != 0)
+            if ($rate != "0")
             {
                 $rate = $cfg["max_upload_rate"];
             }
@@ -122,7 +122,7 @@ if(!empty($torrent))
         $drate = getRequestVar('drate');
         if (empty($drate))
         {
-            if ($drate != 0)
+            if ($drate != "0")
             {
                 $drate = $cfg["max_download_rate"];
             }
@@ -141,7 +141,7 @@ if(!empty($torrent))
         $maxuploads = getRequestVar('maxuploads');
         if (empty($maxuploads))
         {
-            if ($maxuploads != 0)
+            if ($maxuploads != "0")
             {
                 $maxuploads = $cfg["max_uploads"];
             }
@@ -164,7 +164,9 @@ if(!empty($torrent))
         $sharekill = getRequestVar('sharekill');
 
         if ($runtime == "True" )
+        {
             $sharekill = "-1";
+        }
 
         if (empty($sharekill))
         {
@@ -192,7 +194,7 @@ if(!empty($torrent))
             }
         }
 
-        $torrent = urldecode($torrent);
+        //$torrent = urldecode($torrent);
         $alias = getAliasName($torrent);
         $owner = getOwner($torrent);
 
@@ -241,15 +243,15 @@ if(!empty($torrent))
 
         if (usingTornado())
         {
-            $command = $runtime." ".$sharekill." ".$cfg["torrent_file_path"].$alias.".stat ".$owner." --responsefile '".$cfg["torrent_file_path"].$torrent."' --display_interval 5 --max_download_rate ". $drate ." --max_upload_rate ".$rate." --max_uploads ".$maxuploads." --minport ".$minport." --maxport ".$maxport." --rerequest_interval ".$rerequest." --super_seeder ".$superseeder;
+            $command = escapeshellarg($runtime)." ".escapeshellarg($sharekill)." '".$cfg["torrent_file_path"].$alias.".stat' ".$owner." --responsefile '".$cfg["torrent_file_path"].$torrent."' --display_interval 5 --max_download_rate ". escapeshellarg($drate) ." --max_upload_rate ".escapeshellarg($rate)." --max_uploads ".escapeshellarg($maxuploads)." --minport ".escapeshellarg($minport)." --maxport ".escapeshellarg($maxport)." --rerequest_interval ".escapeshellarg($rerequest)." --super_seeder ".escapeshellarg($superseeder);
 
             if(file_exists($cfg["torrent_file_path"].$alias.".prio")) {
                 $priolist = explode(',',file_get_contents($cfg["torrent_file_path"].$alias.".prio"));
                 $priolist = implode(',',array_slice($priolist,1,$priolist[0]));
-                $command .= " --priority ".$priolist;
+                $command .= " --priority ".escapeshellarg($priolist);
             }
 
-            $command .= " ".$cfg["cmd_options"]." > /dev/null &";
+            $command .= " ".escapeshellarg($cfg["cmd_options"])." > /dev/null &";
 
             if ($cfg["AllowQueing"] && $queue == "1")
             {
@@ -271,12 +273,12 @@ if(!empty($torrent))
 
                 if (!$cfg["debugTorrents"])
                 {
-                    $pyCmd = $cfg["pythonCmd"] . " -OO";
+                    $pyCmd = escapeshellarg($cfg["pythonCmd"]) . " -OO";
                 }else{
-                    $pyCmd = $cfg["pythonCmd"];
+                    $pyCmd = escapeshellarg($cfg["pythonCmd"]);
                 }
 
-                $command = "cd " . $cfg["path"] . $owner . "; HOME=".$cfg["path"]."; export HOME; nohup " . $pyCmd . " " .$cfg["btphpbin"] . " " . $command;
+                $command = "cd " . $cfg["path"] . $owner . "; HOME=".$cfg["path"]."; export HOME; nohup " . $pyCmd . " " .escapeshellarg($cfg["btphpbin"]) . " " . $command;
             }
 
         }
@@ -289,7 +291,7 @@ if(!empty($torrent))
         }
 
         // write the session to close so older version of PHP will not hang
-        session_write_close("TorrentFlux");
+        session_write_close();
 
         if($af->running == "3")
         {
@@ -300,7 +302,7 @@ if(!empty($torrent))
         else
         {
             // The following command starts the torrent running! w00t!
-            $result = exec($command);
+            passthru($command);
 
             AuditAction($cfg["constants"]["start_torrent"], $torrent."<br>Die:".$runtime.", Sharekill:".$sharekill.", MaxUploads:".$maxuploads.", DownRate:".$drate.", UploadRate:".$rate.", Ports:".$minport."-".$maxport.", SuperSeed:".$superseeder.", Rerequest Interval:".$rerequest);
 
@@ -394,7 +396,7 @@ if(! $url_upload == '')
         if (is_file($cfg["torrent_file_path"].$file_name))
         {
             // Error
-            $messages .= "<b>Error</b> with (<b>".$file_name."</b>), the file already exists on the server.<br><center><a href=\"".$_SERVER['PHP_SELF']."\">[Refresh]</a></center>";
+            $messages .= "<b>Error</b> with (<b>".htmlentities($file_name)."</b>), the file already exists on the server.<br><center><a href=\"".$_SERVER['PHP_SELF']."\">[Refresh]</a></center>";
             $ext_msg = "DUPLICATE :: ";
         }
         else
@@ -407,7 +409,7 @@ if(! $url_upload == '')
     }
     else
     {
-        $messages .= "<b>Error</b> Getting the File (<b>".$file_name."</b>), Could be a Dead URL.<br><center><a href=\"".$_SERVER['PHP_SELF']."\">[Refresh]</a></center>";
+        $messages .= "<b>Error</b> Getting the File (<b>".htmlentities($file_name)."</b>), Could be a Dead URL.<br><center><a href=\"".$_SERVER['PHP_SELF']."\">[Refresh]</a></center>";
     }
 
     if ($messages == "")
@@ -440,7 +442,7 @@ if(!empty($_FILES['upload_file']['name']))
             if (is_file($cfg["torrent_file_path"].$file_name))
             {
                 // Error
-                $messages .= "<b>Error</b> with (<b>".$file_name."</b>), the file already exists on the server.<br><center><a href=\"".$_SERVER['PHP_SELF']."\">[Refresh]</a></center>";
+                $messages .= "<b>Error</b> with (<b>".htmlentities($file_name)."</b>), the file already exists on the server.<br><center><a href=\"".$_SERVER['PHP_SELF']."\">[Refresh]</a></center>";
                 $ext_msg = "DUPLICATE :: ";
             }
             else
@@ -455,7 +457,7 @@ if(!empty($_FILES['upload_file']['name']))
                 }
                 else
                 {
-                    $messages .= "<font color=\"#ff0000\" size=3>ERROR: File not uploaded, file could not be found or could not be moved:<br>".$cfg["torrent_file_path"] . $file_name."</font><br>";
+                    $messages .= "<font color=\"#ff0000\" size=3>ERROR: File not uploaded, file could not be found or could not be moved:<br>".$cfg["torrent_file_path"] . htmlentities($file_name)."</font><br>";
                 }
             }
         }
@@ -478,10 +480,10 @@ if(!empty($_FILES['upload_file']['name']))
 
 
 // if a file was set to be deleted then delete it
-$delfile = getRequestVar('delfile');
+$delfile = SecurityClean(getRequestVar('delfile'));
 if(! $delfile == '')
 {
-    $alias_file = getRequestVar('alias_file');
+    $alias_file = SecurityClean(getRequestVar('alias_file'));
     if (($cfg["user"] == getOwner($delfile)) || IsAdmin())
     {
         @unlink($cfg["torrent_file_path"].$delfile);
@@ -506,13 +508,13 @@ if(! $delfile == '')
 
 // Did the user select the option to kill a running torrent?
 $kill = getRequestVar('kill');
-if(! $kill == '')
+if(! $kill == '' && is_numeric($kill) )
 {
     include_once("AliasFile.php");
     include_once("RunningTorrent.php");
 
     $kill_torrent = getRequestVar('kill_torrent');
-    $alias_file = getRequestVar('alias_file');
+    $alias_file = SecurityClean(getRequestVar('alias_file'));
     // We are going to write a '0' on the front of the stat file so that
     // the BT client will no to stop -- this will report stats when it dies
     $the_user = getOwner($kill_torrent);
@@ -556,7 +558,7 @@ if(! $kill == '')
     if (!empty($return))
     {
         sleep(3);
-        $result = exec("kill ".$kill);
+        passthru("kill ".$kill);
         // try to remove the pid file
         @unlink($cfg["torrent_file_path"].$alias_file.".pid");
         header("location: ".$return.".php?op=queueSettings");
@@ -572,7 +574,7 @@ if(! $kill == '')
 // Did the user select the option to remove a torrent from the Queue?
 if(isset($_REQUEST["dQueue"]))
 {
-    $alias_file = getRequestVar('alias_file');
+    $alias_file = SecurityClean(getRequestVar('alias_file'));
     $QEntry = getRequestVar('QEntry');
 
     // Is the Qinfo file still there?
@@ -640,11 +642,20 @@ $drivespace = getDriveSpace($cfg["path"]);
     <link rel="icon" href="images/favicon.ico" type="image/x-icon" />
     <link rel="shortcut icon" href="images/favicon.ico" type="image/x-icon" />
     <LINK REL="StyleSheet" HREF="themes/<?php echo $cfg["theme"] ?>/style.css" TYPE="text/css">
-    <META HTTP-EQUIV="Pragma" CONTENT="no-cache; charset=<?php echo _CHARSET ?>">
+    <META HTTP-EQUIV="Pragma" CONTENT="no-cache" charset="<?php echo _CHARSET ?>">
 <?php
     if(!isset($_SESSION['prefresh']) || ($_SESSION['prefresh'] == true))
     {
         echo "<meta http-equiv=\"REFRESH\" content=\"".$cfg["page_refresh"].";URL=index.php\">";
+?>
+<script language="JavaScript">
+    var var_refresh = <?php echo $cfg["page_refresh"] ?>;
+    function UpdateRefresh() {
+        span_refresh.innerHTML = String(var_refresh--);
+        setTimeout("UpdateRefresh();", 1000);
+    }
+</script>
+<?php
     }
 ?>
 <div id="overDiv" style="position:absolute;visibility:hidden;z-index:1000;"></div>
@@ -675,7 +686,7 @@ function ConfirmDelete(file)
 </script>
 </head>
 
-<body topmargin="8" bgcolor="<?php echo $cfg["main_bgcolor"] ?>">
+<body onLoad="UpdateRefresh();" topmargin="8" bgcolor="<?php echo $cfg["main_bgcolor"] ?>">
 
 <div align="center">
 
@@ -752,12 +763,13 @@ if ($cfg["enable_search"])
         <b><?php echo _TORRENTLINKS ?>:</b><br>
 <?php
         $arLinks = GetLinks();
-        foreach($arLinks as $link)
+        if (is_array($arLinks))
         {
-            $arURL = parse_url($link);
-            echo "<a href=\"".$link."\" target=\"_blank\"><img src=\"images/arrow.gif\" width=9 height=9 title=\"".$link."\" border=0 align=\"baseline\">".$arURL['host']."</a><br>\n";
+            foreach($arLinks as $link)
+            {
+                echo "<a href=\"".$link['url']."\" target=\"_blank\"><img src=\"images/arrow.gif\" width=9 height=9 title=\"".$link['url']."\" border=0 align=\"baseline\">".$link['sitename']."</a><br>\n";
+            }
         }
-
         echo "</ul></td>";
 
         $arUsers = GetUsers();
@@ -874,7 +886,7 @@ if ($cfg["enable_search"])
         if ($cfg["enable_torrent_download"])
         {
         ?>
-        <td>&nbsp;&nbsp;&nbsp;<img src="images/down.gif" width="9" height="9" title="<?php echo _DELQUEUE ?>"></td>
+        <td>&nbsp;&nbsp;&nbsp;<img src="images/down.gif" width="9" height="9" title="Download Torrent meta file"></td>
         <td class="tiny">Download Torrent</td>
         <?php
         }
@@ -909,7 +921,7 @@ if ($cfg["enable_search"])
 
     if(!isset($_SESSION['prefresh']) || ($_SESSION['prefresh'] == true))
     {
-        echo "*** "._PAGEWILLREFRESH." ".$cfg["page_refresh"]." "._SECONDS." ***<br>";
+        echo "*** "._PAGEWILLREFRESH." <span id='span_refresh'>".$cfg["page_refresh"]."</span> "._SECONDS." ***<br>";
         echo "<a href=\"".$_SERVER['PHP_SELF']."?pagerefresh=false\"><font class=\"tiny\">"._TURNOFFREFRESH."</font></a>";
     }
     else
@@ -950,9 +962,9 @@ if ($cfg["enable_search"])
                 <td class="tiny" align="right"><?php echo _SERVERLOAD ?>:</td>
                 <td class="tiny">
         <?php
-            if ($cfg["show_server_load"] && isFile($cfg["loadavg_path"]))
+            if ($cfg["show_server_load"] && @isFile($cfg["loadavg_path"]))
             {
-                $loadavg_array = explode(" ", exec("cat ".$cfg["loadavg_path"]));
+                $loadavg_array = explode(" ", exec("cat ".escapeshellarg($cfg["loadavg_path"])));
                 $loadavg = $loadavg_array[2];
                 echo "<strong>".$loadavg."</strong>";
             }
