@@ -21,10 +21,11 @@
     along with TorrentFlux; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
-
 /*
     v 1.01 - Changed Main Categories. (removed TV, changed Movies to Video)
     v 1.02 - Mar 19, 06 - Updated pageing.
+    v 1.03 - May 16, 06 - They changed there URL for Searching back to their domain.
+    v 1.04 - Sep 21, 06 - Fix for Bug found by batmark
 */
 
 class SearchEngine extends SearchEngineBase
@@ -33,12 +34,12 @@ class SearchEngine extends SearchEngineBase
     function SearchEngine($cfg)
     {
         $this->mainURL = "torrentspy.com";
-        $this->altURL = "ts.searching.com";
+        $this->altURL = "www.torrentspy.com";
         $this->mainTitle = "TorrentSpy";
         $this->engineName = "TorrentSpy";
 
         $this->author = "kboy";
-        $this->version = "1.02";
+        $this->version = "1.04";
         $this->updateURL = "http://www.torrentflux.com/forum/index.php/topic,874.0.html";
         $this->Initialize($cfg);
     }
@@ -229,11 +230,11 @@ class SearchEngine extends SearchEngineBase
 
             if ( $this->pg != '' && array_key_exists("db",$_REQUEST))
             {
-                $request = '/search.asp?query=' . $searchTerm . '&pg=' . $this->pg . '&db=' . $_REQUEST["db"] . '&submit.x=24&submit.y=10';
+                $request = '/search.asp?h=&query=' . $searchTerm . '&pg=' . $this->pg . '&db=' . $_REQUEST["db"] . '&submit.x=24&submit.y=10';
             }
             else
             {
-                $request = '/search.asp?query=' . $searchTerm . '&submit.x=24&submit.y=10';
+                $request = '/search.asp?h=&query=' . $searchTerm . '&submit.x=24&submit.y=10';
             }
         }
 
@@ -281,7 +282,7 @@ class SearchEngine extends SearchEngineBase
                         $ts = new tSpy($value);
 
                         // Determine if we should build this output
-                        if (is_int(array_search($ts->MainId,$this->catFilter)))
+                        if (is_int(array_search($ts->MainCategory,$this->catFilter)))
                         {
                             $buildLine = false;
                         }
@@ -415,7 +416,7 @@ class tSpy
 
             if(count($tmpListArr) > 5)
             {
-                $tmpListArr["0"];  // Torrent Name, Download Link, Status
+                //$tmpListArr["0"];  // Torrent Name, Download Link, Status
 
                 $this->torrentDisplayName = $this->cleanLine($tmpListArr["0"]);  // TorrentName
 
